package com.liang.sca.goods.controller;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.lang.generator.SnowflakeGenerator;
import cn.hutool.core.util.IdUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.liang.sca.goods.mapper.ProductSkuMapper;
import com.liang.sca.goods.mapper.ProductSpuMapper;
import com.liang.sca.goods.mapper.ShopProductMapper;
import com.liang.sca.goods.model.ProductSpu;
import com.liang.sca.goods.model.ShopProduct;
import com.liang.sca.goods.model.type.ShopProductInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.elasticsearch.core.ElasticsearchOperations;
import org.springframework.data.elasticsearch.core.IndexOperations;
import org.springframework.data.elasticsearch.core.document.Document;
import org.springframework.data.elasticsearch.core.mapping.IndexCoordinates;
import org.springframework.data.elasticsearch.core.query.IndexQuery;
import org.springframework.data.elasticsearch.core.query.IndexQueryBuilder;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/shop")
public class ShopController {

    @Autowired
    private ElasticsearchOperations elasticsearchOperations;

    @Autowired
    private ShopProductMapper shopProductMapper;

    @Autowired
    private ProductSpuMapper productSpuMapper;

    @Autowired
    private ProductSkuMapper productSkuMapper;

    @RequestMapping("save-es/batch")
    public String saveEsBatch() {
        Snowflake snowflake = IdUtil.getSnowflake();
        List<ShopProduct> shopProducts = shopProductMapper.selectList(new QueryWrapper<>());
        for (ShopProduct shopProduct : shopProducts) {
            ShopProductInfo shopProductInfo = new ShopProductInfo();
            shopProductInfo.setId(shopProduct.getShopId().toString() + shopProduct.getShopSpu().toString());
            shopProductInfo.setShopId(shopProduct.getShopId().toString());
            shopProductInfo.setShopName(shopProduct.getShopName());
            shopProductInfo.setSpuId(shopProduct.getShopSpu().toString());
            shopProductInfo.setProductName(shopProduct.getShopName());
            shopProductInfo.setStore(shopProduct.getShopStore());
            shopProductInfo.setSpuPrice(shopProduct.getShopPrice());
            ProductSpu productSpu = productSpuMapper.selectById(shopProduct.getShopSpu());
            shopProductInfo.setProductName(productSpu.getSpuName());
            shopProductInfo.setSalesCount(productSpu.getSpuSalesCount());
            shopProductInfo.setScore(productSpu.getSpuScore());
            shopProductInfo.setScoreTwo(productSpu.getSpuScoreTwo());
            List<Long> skus = shopProductMapper.skusByShopAndSpu(shopProduct.getShopId(), shopProduct.getShopSpu());
            String shopSku = JSONUtil.toJsonStr(skus);
            shopProductInfo.setSkus(shopSku);

            // 获取索引操作对象
            IndexOperations indexOps = elasticsearchOperations.indexOps(ShopProductInfo.class);
            // 检查索引是否存在
            if (!indexOps.exists()) {
                // 如果索引不存在，则创建索引
                indexOps.create();
            }
            // 创建文档

            // 创建索引查询
            IndexQuery indexQuery = new IndexQueryBuilder()
                    .withId(shopProductInfo.getId().toString())
                    .withObject(shopProductInfo)
                    .build();
            // 执行索引操作
            elasticsearchOperations.index(indexQuery, IndexCoordinates.of("shop_product_info"));
        }
        return "saveEsBatch";
    }
}
